from wildcat.solver.base_solver import BaseSolver


class TSPSolver(BaseSolver):

    def __init__(self, distance=None):
        super().__init__()
        self.distance = distance
        if not (self.distance is None):
            self._build_ising_hamiltonian()

    def _build_ising_hamiltonian(self):
        pass



from wildcat.solver.base_solver import BaseSolver


class QuboSolver(BaseSolver):
    def __init__(self, qubo=None):
        super().__init__()
        if not (qubo is None):
            self.qubo = qubo
        self.build_ising_interactions()




import numpy as np

from wildcat.network.endpoint import Endpoint


class BaseSolver:
    def __init__(self):
        self.ising_interactions = np.zeros(0)
        self.qubo = np.zeros(0)
        self.endpoint = Endpoint()

    def solve(self, callback, endpoint=None):
        self.endpoint = endpoint or self.endpoint
        return self.endpoint.dispatch(solver=self, path=Endpoint.ising_solver_path(), callback=callback)

    def build_qubo(self):
        self.qubo = -self.ising_interactions.copy() * 4
        for i in range(self.qubo.shape[0]):
            self.qubo[i][i] = 2 * (self.ising_interactions[i].sum() + self.ising_interactions.sum(axis=0)[i]) - 6 * \
                              self.ising_interactions[i][i]

    def build_ising_interactions(self):
        self.ising_interactions = -self.qubo.copy() / 4.0
        for i in range(self.ising_interactions.shape[0]):
            self.ising_interactions[i][i] = - (self.qubo[i].sum() + self.qubo.sum(axis=0)[i]) / 4.0




import numpy as np

def symmetrize(a):
    return a + a.T - np.diag(a.diagonal())


def check_symmetric(a, tol=1e-8):
    return np.allclose(a, a.T, atol=tol)

def random_symmetric_matrix(size=100):
    return symmetrize(np.random.uniform(-1, 1, (size, size)))